//
//  IdeaListTableViewController.m
//  dreamToGoals
//
//  Created by Nathan Fair on 5/11/11.
//  Copyright 2011 computer science undergraduate association. All rights reserved.
//

#import "IdeaListTableViewController.h"
#import "Idea.h"
#import "IdeaViewController.h"

@implementation IdeaListTableViewController

@synthesize ideasArray, managedObjectContext, addButton, someRefForTesting;


-(void) addIdea {

	/*
	// adds a managed object to the managed object context and returns a propertly initialized instance of the correct class
	Idea * idea = (Idea *)[NSEntityDescription insertNewObjectForEntityForName:@"Idea" inManagedObjectContext:managedObjectContext];
	
	[idea setCreationDate:[NSDate date]];
	
	// for now all ideas will be the same
	[idea setTitle:@"A New Idea"];
	
	// save the context pushes all changes from context to persistant store
	NSError * error = nil;
	if(![managedObjectContext save:&error]){
		// handle the error (but how !?!?)
		
	
	 }
	
	// add the new idea to the list of ideas
	[ideasArray insertObject:idea atIndex:0];
	NSIndexPath * indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
	[self.tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPath]
						  withRowAnimation:UITableViewRowAnimationFade];
	[self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0] 
						  atScrollPosition:UITableViewScrollPositionTop animated:YES];
	
	 */
	IdeaViewController * ideaCreation = [[IdeaViewController alloc] initWithNibName:@"IdeaViewController" bundle:nil];
	[ideaCreation setManagedObjectContext:self.managedObjectContext];
	[self.navigationController pushViewController:ideaCreation animated:YES];
	[ideaCreation release];


}

#pragma mark -
#pragma mark Initialization

/*
- (id)initWithStyle:(UITableViewStyle)style {
    // Override initWithStyle: if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization.
    }
    return self;
}
*/


#pragma mark -
#pragma mark View lifecycle


- (void)viewDidLoad {
    [super viewDidLoad];
	
	// set title
	self.title = @"Ideas";
	
	// set up add button
	
	addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
															  target:self
															  action:@selector(addIdea)];
	addButton.enabled = YES;
	self.navigationItem.rightBarButtonItem = addButton;
	
	
	// fetch saved ideas
	NSFetchRequest * request = [[NSFetchRequest alloc] init];
	NSEntityDescription * entity = [NSEntityDescription entityForName: @"Idea" inManagedObjectContext:managedObjectContext];
	[request setEntity:entity];
	
	// sort ordering (by date)
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"creationDate" ascending:NO];
	NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
	[request setSortDescriptors:sortDescriptors];
	[sortDescriptors release];
	[sortDescriptor release];
	

	// the actual fetch
	NSError *error = nil;
	
	NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
	
	if (mutableFetchResults == nil) {
		
		// Handle the error.
		
	}
	[self setIdeasArray:mutableFetchResults];
	[mutableFetchResults release];
	[request release];
	
}

- (void)viewWillAppear:(BOOL)animated {
	
	[super viewWillAppear:animated];
	[self.tableView reloadData];
}


/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/
/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations.
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/


#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return [ideasArray count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	// A date formatter for the time stamp.
	
    static NSDateFormatter *dateFormatter = nil;
	
    if (dateFormatter == nil) {
		
        dateFormatter = [[NSDateFormatter alloc] init];
		
        [dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
		
        [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
		
    }
	
	
    static NSString *CellIdentifier = @"Cell";
    
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
		cell.textLabel.lineBreakMode = UILineBreakModeWordWrap;
		cell.textLabel.numberOfLines = 0;
		cell.textLabel.font = [UIFont fontWithName:@"Helvetica" size:17.0];
	}
    
    // Configure the cell...
	
	Idea * idea = (Idea *)[ideasArray objectAtIndex:indexPath.row];
	cell.textLabel.text = [idea title];
	cell.detailTextLabel.text = [dateFormatter stringFromDate:[idea creationDate]];
    
    return cell;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath{
	
    if (editingStyle == UITableViewCellEditingStyleDelete) {
		
		
		
        // Delete the managed object at the given index path.
		
        NSManagedObject *ideaToDelete = [ideasArray objectAtIndex:indexPath.row];
		[managedObjectContext deleteObject:(NSManagedObject *)[(Idea *)ideaToDelete Notes]];
        [managedObjectContext deleteObject:ideaToDelete];
		
		
		
        // Update the array and table view.
		
        [ideasArray removeObjectAtIndex:indexPath.row];
		
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
		
		
		
        // Commit the change.
		
        NSError *error = nil;
		
        if (![managedObjectContext save:&error]) {
			
            // Handle the error.
			
        }
		
    }
	
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
	Idea * idea = (Idea *)[ideasArray objectAtIndex:indexPath.row];
    NSString *cellText = [idea title];
    UIFont *cellFont = [UIFont fontWithName:@"Helvetica" size:17.0];
    CGSize constraintSize = CGSizeMake(280.0f, MAXFLOAT);
    CGSize labelSize = [cellText sizeWithFont:cellFont constrainedToSize:constraintSize lineBreakMode:UILineBreakModeWordWrap];
	
    return labelSize.height + 20;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source.
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Navigation logic may go here. Create and push another view controller.
    Idea * anIdea = [ideasArray objectAtIndex:indexPath.row];
	
	IdeaViewController * ideaCreation = [[IdeaViewController alloc] initWithNibName:@"IdeaViewController" bundle:nil];
	[ideaCreation setIdea:anIdea];
	if (anIdea.Notes) {
		[ideaCreation setNote:anIdea.Notes];
	}
	[ideaCreation setManagedObjectContext:managedObjectContext];
	[self.navigationController pushViewController:ideaCreation animated:YES];
	[ideaCreation release];

	
    
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    self.ideasArray = nil;
	self.addButton = nil;
}


- (void)dealloc {
	[managedObjectContext release];
	[ideasArray release];
	[addButton release];
    [super dealloc];
}


@end

