//
//  IdeaViewController.m
//  dreamToGoals
//
//  Created by Nathan Fair on 5/11/11.
//  Copyright 2011 computer science undergraduate association. All rights reserved.
//

#import "IdeaViewController.h"
#import "Notes.h"
#import "IdeaListTableViewController.h"
#import <QuartzCore/QuartzCore.h>

@implementation IdeaViewController

@synthesize idea, subject, body, doneButton, managedObjectContext, note;

- (void) stopBodyEdit{
	[self.body resignFirstResponder];
	self.navigationItem.rightBarButtonItem = nil;
}

// The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
/*
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization.
    }
    return self;
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

	
	if (idea == nil) {
		self.title = @"New Idea";
	}
	if (idea.title) {
		subject.text = [idea title];
	}
	
	if (note) {
		[self.body setText:[note discription]];
	}
	
	
	// configure the done button. when we start editing the textview, add it to the UIBar
	doneButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(stopBodyEdit)];
	doneButton.enabled = YES;
	
	body.layer.cornerRadius = 5;
	
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations.
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

#pragma mark -
#pragma mark UITextFieldDelegate

- (BOOL)textFieldShouldReturn:(UITextField *)textField{
	[textField resignFirstResponder];
	return YES;
}

#pragma mark -
#pragma mark UITextViewDelegate

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range 
 replacementText:(NSString *)text
{
	// check to see if it is the subject textView
	if (subject == textView) {
	
    // Any new character added is passed in as the "text" parameter
    if ([text isEqualToString:@"\n"]) {
        // Be sure to test for equality using the "isEqualToString" message
        [textView resignFirstResponder];
		
        // Return FALSE so that the final '\n' character doesn't get added
        return FALSE;
    }
    
	}
	// For any other character return TRUE so that the text gets added to the view
    return TRUE;
}

- (void)textViewDidBeginEditing:(UITextView *)textView{
	// I will add a done button to the  page so that we can be done with edititing
	self.navigationItem.rightBarButtonItem = doneButton;
}

- (BOOL)textViewShouldEndEditing:(UITextView *)textView{
	NSLog(@"textviewshouldreturn triggerd");
	return YES;
}

#pragma mark - 
#pragma mark UIViewController memory managemtent

- (void)viewWillDisappear:(BOOL)animated{
	// first off test whether we have something to save.
	if (self.body.text == nil) {
		[self.body setText:@""];
	}
	if (self.subject.text == nil) {
		[self.subject setText:@""];
	}
	
	
	if ([self.subject.text isEqualToString:@""] && [self.body.text isEqualToString:@""]){
		NSLog(@"we aren't going to add or change anything in the datamodel");
	}
	else {
		if ([self.subject.text isEqualToString:@""]){
			[self.subject setText:@"no subject"];
		}


		// if we already had data in the data model then we update otherwise we are going to create one
		
		if (idea == nil) {
			// now it is time to create the idea and insert it into the datamodel as well as inserting it into 
			// the IdeaListTableViewController's data source :)
			
			// adds a managed object to the managed object context and returns a propertly initialized instance of the correct class
			idea = (Idea *)[NSEntityDescription insertNewObjectForEntityForName:@"Idea" inManagedObjectContext:managedObjectContext];
			
			// because we both create an autoreleased Idea and we pass in a retained Idea to the same variable it is important that
			// they have the same retain count.  So I will increment this idea's retain count. note if I had used the setter instead
			// of this local version I would have avoided this error
			[idea retain];
			
			[idea setCreationDate:[NSDate date]];
			[idea setTitle:self.subject.text];
			
			// creation of the associated Notes
			
			[self setNote:[NSEntityDescription insertNewObjectForEntityForName:@"Notes" inManagedObjectContext:managedObjectContext]];
			// create relations
			idea.Notes = note;
			
			note.discription = self.body.text;
			
			// save the context pushes all changes from context to persistant store
			NSError * error = nil;
			if(![managedObjectContext save:&error]){
				// handle the error (but how !?!?)
			}
			NSArray * controllers = self.navigationController.viewControllers;
			// it would be an index offset of 2 but the current view controller has already been poped from the stack... 
			IdeaListTableViewController * listController = [controllers objectAtIndex:[controllers count] -1];
			[listController.ideasArray insertObject:idea atIndex:0];
			
			
		}
		else {
			
			
			[idea setCreationDate:[NSDate date]];
			[idea setTitle:self.subject.text];
			if (idea.Notes == nil) {
				[self setNote:[NSEntityDescription insertNewObjectForEntityForName:@"Notes" inManagedObjectContext:managedObjectContext]];
				// create relations
				idea.Notes = note;
				
			}
			note.discription = self.body.text;
			
			
			
			// save the context pushes all changes from context to persistant store
			NSError * error = nil;
			if(![managedObjectContext save:&error]){
				// handle the error (but how !?!?)
				
				
			}
		}
		
		
	}
	
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
	self.managedObjectContext = nil;
	self.subject = nil;
	self.body = nil;
	self.doneButton = nil;
}


- (void)dealloc {
	[idea release];
	[managedObjectContext release];
	[subject release];
	[body	 release];
	[doneButton release];
	
    [super dealloc];
}


@end
